#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os.path
import inspect
import logging

from ply import lex, yacc


class PLYParser(object):
    """Wrapper object for PLY lexer/parser."""

    def _table_name(self, suffix, relative=False):
        """Return the module name for PLY's parsetab file."""
        mname = inspect.getmodule(self.__class__).__name__ + '_' + suffix
        if relative:
            mname = mname.split('.')[-1]
        return mname

    def _write_tables(self):
        """Write parser table (for distribution purposes)."""
        path = inspect.getfile(self.__class__)
        parent = os.path.split(path)[0]
        # Need to change directories to get the file written at the right
        # location.
        cwd = os.getcwd()
        os.chdir(parent)
        tabname = self._table_name('lex', relative=True)
        lex.lex(object=self, lextab=tabname, optimize=True, debug=False)
        tabname = self._table_name('tab', relative=True)
        yacc.yacc(module=self, tabmodule=tabname, optimize=True, debug=False)
        os.chdir(cwd)

    def parse(self, input, fname=None, debug=False):
        optimize = not debug
        tabname = self._table_name('lex')
        lexer = lex.lex(object=self, lextab=tabname,
                        optimize=optimize, debug=debug)
        if hasattr(input, 'read'):
            input = input.read()
        lexer.input(input)
        tabname = self._table_name('tab')
        parser = yacc.yacc(module=self, tabmodule=tabname,
                           optimize=optimize, debug=debug)
        if debug:
            logger = logging.getLogger()
        else:
            logger = yacc.NullLogger()
        parsed = parser.parse(lexer=lexer, tracking=True, debug=logger)
        return parsed



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper
from cli.messages import Messages


class UpdateCommand(OvirtCommand):

    name = 'update'
    description = 'update an object'
    args_check = 2
    valid_options = [ ('*', str) ]

    helptext = """\
        == Usage ==

        update <type> <id> [parent identifiers] [command options]

        == Description ==

        Update an existing object. This command requires the following
        arguments:

          * type        The type of object to update.
          * id          The identifier of the object to update

        == Supported Help formats ==

        - This help will list all available attribute options for given resource update
          
          * format      - help update type
          * example     - help update vm

        - This help will list all available attribute options for given subresource update
          
          * format      - help update type --parentid
          * example     - help update disk --vm-identifier myvm

        == Available Types ==

        The following object types are available:

          $types

        == Base Identifiers ==

        Some objects can only exist inside other objects. For example a disk
        can only exist as part of a virtual machine. In this case you want to
        update such an object, one or more base identifier options need to be
        given to identify the containing object. These options have the form
        --<type>id <id>.

        == Attribute Options ==

        Attribute options specifiy values for attributes of the object that is
        to be updated.

        Type 'help update <type>' to see an overview of which attributes are
        available for a specific type.

        == Examples ==

        - This example updates a virtual machine with name "myvm", setting new 
          description and amount of monitors.

          $ update vm myvm --display-monitors 1 --description test1

        - This example updates a virtual machine disk with name "mydisk", setting 
          "bootable" to true.

          $ update disk "mydisk" --vm-identifier "myvm" --bootable true

        == Return Values ==

          $statuses
        """

    helptext1 = """\
        == Usage ==

        update <type> <id> [parent identifiers] [command options]

        == Description ==

        Update an existing object. This command requires the following
        arguments:

          * type        The type of object to update.
          * id          The identifier of the object to update

        See 'help update' for generic help on creating objects.

        == Attribute Options ==

        The following options are available for objects with type '$type':

          $options

        == Collection based option format ==

          * [--x-y: collection]
            {
              [y.a: string]
              [y.b: string]
              [y.c: string]
            }

          e.g:

          --x-y "y.a=a1,y.b=b1,y.c=c1"
          --x-y "y.a=a2,y.b=b2,y.c=c2"
          ...

          where a, b, c are option properties and aN, bN, cN is actual user's data

        == Return Values ==

          $statuses
        """

    def execute(self):
        """Execute the "update" command."""
        args = self.arguments
        opts = self.options

        typs = TypeHelper.get_types_containing_method(
                      'update',
                       expendNestedTypes=True,
                       groupOptions=True
        )

        resource = self.get_object(
                       args[0], args[1],
                       self.resolve_base(opts),
                       context_variants=typs[args[0]]
        )

        if resource is None:
            self.error(
               Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
            )
        elif hasattr(resource, 'update'):
            obj = self.update_object_data(self.__create_set_superclass(resource), opts)
            result = self.execute_method(obj, 'update', opts)
        else:
            self.error(
               Messages.Error.OBJECT_IS_IMMUTABLE % (args[0], args[1])
            )
        self.context.formatter.format(self.context, result)

    def __create_set_superclass(self, resource):
        """Create an instance of param object."""
        param_obj = type(resource.superclass).factory()
        if hasattr(param_obj, 'id'):
            setattr(param_obj, 'id', getattr(resource, 'id'))
        resource.superclass = param_obj
        return resource

    def show_help(self):
        """Show help for "update"."""

        self.check_connection()
        args = self.arguments
        opts = self.options

        subst = {}
        types = TypeHelper.get_types_containing_method(
               'update',
               expendNestedTypes=True,
               groupOptions=True
        )
        subst['types'] = self.format_map(types)
        statuses = self.get_statuses()
        subst['statuses'] = self.format_list(statuses)

        if len(args) > 0 and self.is_supported_type(types.keys(), args[0]):
            if len(args) == 2:
                base = self.resolve_base(self.options)
                obj = self.get_object(
                          args[0], args[1],
                          base,
                          context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                          Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )
                helptext = self.helptext1
                params_list = self.get_options(
                           method='update',
                           resource=obj,
                           sub_resource=base,
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)
                subst['type'] = args[0]

            elif len(args) == 1 and len(opts) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]

                options = self.get_options(
                           method='update',
                           resource=args[0],
                           sub_resource=self.resolve_base(self.options),
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(options)
                subst['type'] = args[0]
            elif len(args) == 1:
                helptext = self.helptext
                subst['type'] = args[0]
                subst['types'] = self.format_map({args[0]:types[args[0]]})
            else:
                helptext = self.helptext
        else:
            helptext = self.helptext

        helptext = self.format_help(helptext, subst)
        self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.shell.cmdshell import CmdShell
from ovirtcli.utils.autocompletionhelper import AutoCompletionHelper


class HistoryCmdShell(CmdShell):
    NAME = 'history'
    OPTIONS = [
       'last', 'first'
    ]

    def __init__(self, context, parser):
        CmdShell.__init__(self, context, parser)

    def do_history(self, args):
        return self.context.execute_string(HistoryCmdShell.NAME + ' ' + args + '\n')

    def complete_history(self, text, line, begidx, endidx):
        return AutoCompletionHelper.complete(line=line, text=text,
                                             args={}.fromkeys(HistoryCmdShell.OPTIONS),
                                             all_options=True)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import sys
import os

import itertools

from ovirtcli.utils.typehelper import TypeHelper
from ovirtcli.infrastructure.settings import OvirtCliSettings


class CmdShell(object):
    def __init__(self, context, parser):
        self.__context = context
        self.__parser = parser
        self.__owner = self

    def get_owner(self):
        return self.__owner


    def get_parser(self):
        return self.__parser

    def get_context(self):
        return self.__context

    context = property(get_context, None, None, None)
    parser = property(get_parser, None, None, None)
    owner = property(get_owner, None, None, None)

    ############################ CONFIG #################################
    def copy_environment_vars(self, context):
        """Copy environment variables into configuration variables in the
        execution context."""
        for var in ('url', 'username', 'password'):
            envvar = OvirtCliSettings.PRODUCT + '_%s' % var.upper()
            confvar = 'ovirt-shell:%s' % var
            if envvar in os.environ:
                try:
                    context.settings[confvar] = os.environ[envvar]
                except ValueError, e:
                    sys.stderr.write('error: %s\n' % str(e))
                    return False
        return True


    def copy_cmdline_options(self, options, context, parser):
        """Copy command-line options into configuration variables in the
        execution context."""
        for opt in parser.option_list:
            if not opt.dest:
                continue
            value = getattr(options, opt.dest)
            if value is None:
                continue
            if opt.dest in ('debug', 'verbosity'):
                dest = 'cli:%s' % opt.dest
            else:
                dest = 'ovirt-shell:%s' % opt.dest
            try:
                context.settings[dest] = value
            except KeyError:
                pass
            except ValueError, e:
                sys.stderr.write('error: %s\n' % str(e))
                return False
        return True
    #####################################################################
    def __generate_resource_specific_options__(self, args, line, callback):
        specific_options = {}

        if line:
            spl = line.rstrip().split(' ')
            if len(spl) >= 2:
                obj = spl[1].strip()
                base = self._resolve_base(spl[1:])
                if base:
                    callback(base, specific_options, line=line, key=obj)
                else:
                    callback(obj, specific_options, line=line)
            elif len(spl) == 2 and spl[1] != '' and spl[1].strip() in args.keys():
                callback(spl[1].strip(), specific_options, line=line)

            return specific_options

    def _resolve_base(self, args):
        """resolves a base object from a set of '--type-identifier value' options."""
        PARENT_IDENTIFIER = '-identifier'
        parnet_candidates = [item for item in args
                                      if item.endswith(PARENT_IDENTIFIER)]
        parnet_candidates_permutations = list(itertools.permutations(parnet_candidates))

        if parnet_candidates_permutations[0]:
            for combination in parnet_candidates_permutations:
                candidates = [item[2:-11] for item in combination
                                              if item.endswith(PARENT_IDENTIFIER)]
                candidate = (''.join(candidates) + args[0]).lower()
                dt = TypeHelper.getDecoratorType(candidate)
                if dt: return dt

        return None

    def get_resource_specific_options(self, args, line, callback):
        """
         Generates resource specific options. 
         
         @param args: args to process 
         @param line: line to process
         @param callback: callback to call when done
         
        """
        return self.__generate_resource_specific_options__(args, line, callback)

    def _error(self, msg):
        """
        prints error to stderr

        @param msg: error message
        """
        sys.stderr.write("\nerror: " + msg + "\n")
        sys.stdout.write("\n")



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli.command.command import Command


class HelpCommand(Command):

    name = 'help'
    description = 'show help'
    args_check = None
    valid_options = [('*', str)]

    helptext = """\
        == Usage ==

        help
        help <command> [arguments] [options]

        == Description ==

        Show help about a command. If no command is specified, a general help
        section is displayed.
        """

    helptext1 = """\
        == Introduction ==

        Welcome to cli-test. This is a test driver for the python-cli
        command-line interface construction toolkit. You can type any command
        in the interface below.

        == Available Commands ==

          $commands

        Type 'help <command>' for any of these commands to show detailed help.

        == Command Syntax ==

        The general format for each command is:

          <command> [arguments] [options]

        If arguments contain spaces or other reserved characters, you need to
        quote them. You can use single (') and double (") quotes for this. The
        difference between single and double quotes is that within single
        quotes you can't use any single quotes, while in double quotes you can
        use a double quote provided that you escape it with backslash ('\\').

        Options are always in the long form: --option [value]. The value can
        be optional or mandatory, dependng on the command.

        In addition to the basic command form, the following functionality is
        available:

          * You can use the '<', '<<', '>' and '>>' characters to perform
            shell-like redirections to files in the file system.
          * The output of any command can be piped to a shell command with
            the '|' character.
          * Shell commands can be executed by typing a '!' at the beginning
            of a line.
          * Comments start with '#' and end at the end of a line.
          * Newlines can be escaped by putting a '\\' in front of them. This
            works outside as well as within a quoted string.
          * Multiple commands can be entered on a single line by separating
            them with a semicolon (';').

        == Configuration Variables ==

        A numer of configuration variables are defined that allow you to
        customize the way in which ovirt-shell operations. Type 'show' to see a
        list of all configuration variables and their current values.

        == Examples ==

        Show all configuration variables and redirect the output to a file
        'foo.txt':

          $ set > foo.txt

        The following pages through a long help text:

          $ help | less

        The following shows the contents of a file named foo.txt:

          $ !cat foo.txt
        """

    def execute(self):
        args = self.arguments
        stdout = self.context.terminal.stdout
        if len(args) == 0:
            subst = {}
            commands = self.get_commands()
            subst['commands'] = self.format_list(commands, sort=True)
            helptext = self.format_help(self.helptext1, subst)
            stdout.write(helptext)
        else:
            name = args[0]
            args = args[1:]
            opts = [('--help', None)]
            opts += self.options.items()
            command = self.context._create_command(name, args, opts)
            command.run(self.context)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import sys

from ovirtcli.infrastructure.options import OvirtCliOptionParser
from ovirtcli.infrastructure.context import OvirtCliExecutionContext
from ovirtcli.infrastructure.object import create
from ovirtcli.shell.engineshell import EngineShell

    ############################## MAIN #################################
def main():
    parser = create(OvirtCliOptionParser)
    context = OvirtCliExecutionContext(sys.argv, parser)
    shell = EngineShell(context, parser)

    if len(sys.argv) > 1:
        args = ''
        if len(sys.argv) > 2:
            args = sys.argv[1] + " "
            for item in sys.argv[2:]:
                args += '"' + item + '" '
        else:
            args = ' '.join(sys.argv[1:])
        shell.onecmd_loop(args)
    else:
        shell.onecmd_loop('')
    ########################### __main__ #################################
if __name__ == '__main__':
    main()
    ######################################################################



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from cli.messages import Messages

class AddCommand(OvirtCommand):

    name = 'add'
    aliases = ('add',)
    description = 'creates a new object or adds existent'
    args_check = 1
    valid_options = [ ('*', str) ]

    helptext0 = """\
        == Usage ==

        add <type> [parent identifiers] [command options]

        == Description ==

        Creates a new object or adds existent

        == Supported Help formats ==

        - This help will list all available attribute options for given resource creation
          
          * format      - help add resource_name
          * example     - help add vm

        - This help will list all available attribute options for given subresource creation
          
          * format      - help add subresource_name --resource-identifier xxx
          * example     - help add disk --vm-identifier myvm

        == Available Types ==

        The following types of objects can be added:

          $types

        == Base Identifiers ==

        Some objects can only be created inside other objects. For example a
        disk can only be created inside a virtual machine. In this case, one
        or more base identifier options need to be given to identify the
        containing object. These options have the form --<type>id <id>.

        == Attribute Options ==

        Attribute options specify values for attributes of the to be created
        object.

        Attributes for the new object can be specified in one of two ways.

          * Using command-line options. For example, "--description foo"
            would add the object with a description of "foo".
          * By providing pre-formatted input, in the format specified by the
            'input_format' configuration variable. In this case the input
            needs to be provided using the '<<' input redirection operator.

        Type 'help add <type>' to see an overview of which attributes are
        available for a given type.
          
        == Examples ==

        - This example create a new virtual machine in the Default cluster based on the
          Blank template:

          $ add vm --name myvm --template-name iscsi_desktop_tmpl --cluster-name Default_iscsi
                      
        - This example create vm nic:

          $ add nic --vm-identifier cli_vm3 --network-name engine --name test


        == Return Values ==

          $statuses
        """

    helptext1 = """\
        == Usage ==

        add <type> [parent identifiers] [command options]

        == Description ==

        Creates a new object or adds existent with type $type. See 'help add' for generic
        help on creating objects.

        == Attribute Options ==

        The following options are available for objects with type $type:

          $options

        == Collection based option format ==

          * [--x-y: collection]
            {
              [y.a: string]
              [y.b: string]
              [y.c: string]
            }

          e.g:

          --x-y "y.a=a1,y.b=b1,y.c=c1"
          --x-y "y.a=a2,y.b=b2,y.c=c2"
          ...

          where a, b, c are option properties and aN, bN, cN is actual user's data

        == Return Values ==

          $statuses
        """

    def execute(self):
        """Execute the "add" command."""
        args = self.arguments
        opts = self.options
        base = self.resolve_base(opts)
        typ = args[0] + 's'
        collection = None
        typs = self.get_singular_types(method='add', typ=args[0])

        if base:
            if hasattr(base, typ):
                collection = getattr(base, typ)
        else:
            connection = self.check_connection()
            if hasattr(connection, typ):
                collection = getattr(connection, typ)

        if collection != None:
            result = self.execute_method(collection, 'add', opts)
            self.context.formatter.format(self.context, result)
        else:
            err_str = Messages.Error.CANNOT_CREATE
            if typs:
                err_str = err_str + \
                (Messages.Info.POSSIBALE_ARGUMENTS_COMBINATIONS % str(typs))
            self.error(
                   err_str % (args[0], typ)
            )


    def show_help(self):
        """Show help for "add"."""
        args = self.arguments
        opts = self.options
        types = self.get_singular_types(method='add')
        subst = {}
        if len(args) == 0:
            helptext = self.helptext0
            subst['types'] = self.format_map(types)
        elif len(args) == 1:
            if self.is_supported_type(types.keys(), args[0]):
                helptext = self.helptext1
                params_list = self.get_options(
                       method='add',
                       resource=args[0],
                       sub_resource=self.resolve_base(opts),
                       context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list, sort=False)
                subst['type'] = args[0]
        statuses = self.get_statuses()
        subst['statuses'] = self.format_list(statuses)
        helptext = self.format_help(helptext, subst)
        self.write(helptext)



#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from ovirtcli.utils.colorhelper import ColorHelper
from cli.executionmode import ExecutionMode
from ovirtcli.shell.promptmode import PromptMode
from urlparse import urlparse


class PromptManager(object):
    """
    manages shell prompt
    """
    def __init__(self, engineshell):
        self.__engineshell = engineshell
        self.__org_prompt = ''
        self.__engineshell.prompt = self.set_prompt(
                                    PromptMode.Disconnected
        )

    # PromptMode.Default
    def set_prompt(self, mode):
        self.__engineshell.onPromptChange.fire()
        if mode == PromptMode.Multiline:
            if not self.__org_prompt:
                self.__org_prompt = self.__engineshell.prompt
            self.__engineshell.prompt = '> '
        elif mode == PromptMode.Original:
            if self.__org_prompt:
                self.__engineshell.prompt = self.__org_prompt
                self.__org_prompt = ''
        elif mode == PromptMode.Disconnected or mode == PromptMode.Default:
            if not self.__org_prompt and self.__engineshell.prompt != "(Cmd) ":
                self.__org_prompt = self.__engineshell.prompt
            self.__engineshell.prompt = self.__get_disconnected_prompt()
        elif mode == PromptMode.Connected:
            self.__engineshell.prompt = self.__get_connected_prompt()
        elif mode == PromptMode.Unauthorized:
            self.__engineshell.prompt = self.__get_unauthorized_prompt()

    def __get_unauthorized_prompt(self):
        dprompt = self.__engineshell.context.settings.get('ovirt-shell:ps1.unauthorized')
        if self.__engineshell.context.mode != ExecutionMode.SCRIPT \
           and self.__engineshell.context.interactive:
            dprompt = dprompt.replace(
                          "unauthorized",
                          ColorHelper.colorize(
                                "unauthorized",
                                color=ColorHelper.RED,
                                is_prompt=True
                          )
            )
        return dprompt

    def __get_disconnected_prompt(self):
        dprompt = self.__engineshell.context.settings.get('ovirt-shell:ps1.disconnected')
        if self.__engineshell.context.mode != ExecutionMode.SCRIPT \
           and self.__engineshell.context.interactive:
            dprompt = dprompt.replace(
                          "disconnected",
                          ColorHelper.colorize(
                                "disconnected",
                                color=ColorHelper.RED,
                                is_prompt=True
                          )
            )
        return dprompt

    def __get_connected_prompt(self):
        if self.__engineshell.context.settings.get('ovirt-shell:extended_prompt'):
            url = self.__engineshell.context.settings.get('ovirt-shell:url')
            url_obj = urlparse(url)
            if url_obj and hasattr(url_obj, 'hostname'):
                cprompt = self.__engineshell.context.settings.get(
                               'ovirt-shell:ps3.connected'
                          ) % {
                               'host':url_obj.hostname
                }
                if self.__engineshell.context.mode != ExecutionMode.SCRIPT \
                   and self.__engineshell.context.interactive:
                    cprompt = cprompt.replace(
                              "connected@" + url_obj.hostname,
                              ColorHelper.colorize(
                                    'connected@' + url_obj.hostname,
                                    color=ColorHelper.GREEN,
                                    is_prompt=True
                              )
                )
                return cprompt

        cprompt = self.__engineshell.context.settings.get('ovirt-shell:ps2.connected')
        if self.__engineshell.context.mode != ExecutionMode.SCRIPT \
           and self.__engineshell.context.interactive:
            cprompt = cprompt.replace(
                              "connected",
                              ColorHelper.colorize(
                                 "connected",
                                 color=ColorHelper.GREEN,
                                 is_prompt=True
                               )
        )
        return cprompt



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import datetime
import sys

from ovirtcli.format.format import Formatter

from ovirtsdk.xml import params
from ovirtsdk.infrastructure.common import Base
from ovirtsdk.infrastructure import brokers
import types
from ovirtsdk.xml.params import ApiSummary


class FormatMode():
    FULL, REDUCED = range(2)

    def __init__(self, Type):
        self.value = Type

    def __str__(self):
        if self.value == FormatMode.FULL:
            return 'FULL'
        if self.value == FormatMode.REDUCED:
            return 'REDUCED'

    def __eq__(self, y):
        return self.value == y.value

class TextFormatter(Formatter):
    """Text formatter."""

    name = 'text'

    # list of complex types that should be treated as
    # primitives (e.g should be wrapped to string at runtime)
    complex_type_exceptions = [datetime.datetime]


    def _get_fields(self, typ):
        assert typ is not None
        return typ.__dict__.keys()

    def sort_fields(self, lst, strategy=[]):
        out = sorted(lst)
        for i in range(len(strategy)):
            if strategy[i] in out:
                out.remove(strategy[i])
                out.insert(i, strategy[i])

        return out

    def __get_max_field_width(self, resource, fields_exceptions, width= -1, show_empty=False, resource_context=None, mode=FormatMode.FULL):
        new_field = None
        width0 = width

        for field in self._get_fields(resource):
            if field not in fields_exceptions:
                value = resource.__dict__[field]
                if isinstance(value, list):
                    value_list = value
                    for item in value_list:
                        value = item
                        if hasattr(params, type(value).__name__) or hasattr(brokers, type(value).__name__):
                            if resource_context is not None:
                                width0 = max(width0, self.__get_max_field_width(resource=value,
                                                                                fields_exceptions=fields_exceptions,
                                                                                width=width0,
                                                                                show_empty=show_empty,
                                                                                resource_context=(resource_context + '.' + field)))
                            else:
                                width0 = max(width0, self.__get_max_field_width(resource=value,
                                                                                fields_exceptions=fields_exceptions,
                                                                                width=width0,
                                                                                show_empty=show_empty,
                                                                                resource_context=field))
                            continue
                        if (value == None or value == '') and show_empty == False:
                            continue
                        new_field = field if resource_context is None else resource_context.lower() + '.' + field
                        width0 = max(width0, len(new_field))
                else:
                    value_type = type(value)
                    if (hasattr(params, value_type.__name__) \
                        and value_type not in TextFormatter.complex_type_exceptions) \
                        or hasattr(brokers, value_type.__name__):

                        if resource_context is not None:
                            width0 = max(width0, self.__get_max_field_width(resource=value,
                                                                            fields_exceptions=fields_exceptions,
                                                                            width=width0,
                                                                            show_empty=show_empty,
                                                                            resource_context=(resource_context + '.' + field)))
                        else:
                            width0 = max(width0, self.__get_max_field_width(resource=value,
                                                                            fields_exceptions=fields_exceptions,
                                                                            width=width0,
                                                                            show_empty=show_empty,
                                                                            resource_context=field))
                        continue
                    if (value == None or value == '') and show_empty == False:
                        continue
                    new_field = field if resource_context is None else resource_context.lower() + '.' + field
                    width0 = max(width0, len(new_field))
        return width0

    def __write_context(self, format0, field, value, resource_context, reduced_mode_fields, mode=FormatMode.FULL, resource=None, parent=None):
        context = self.context
        stdout = context.terminal.stdout

        # Statistic values are always represented as floating point numbers,
        # even if they are declared as integers, so in order to render them
        # correctly we need to do a type conversion:
        if type(parent) == params.Values and type(resource) == params.Value:
            if parent.get_type() == "INTEGER":
                value = long(value)

        if type(value) == types.UnicodeType:
            val = value
        else:
            val = str(value)
        fil = (field if resource_context is None
                     else resource_context.lower() + '.' + field)\
              .replace('.', '-')
        fil = fil[:len(fil) - 1] if fil.endswith('_') \
                                     else fil

        if mode != FormatMode.REDUCED or fil in reduced_mode_fields:
            stdout.write(format0 % fil)
            stdout.write(': ')
            stdout.write(val)
            stdout.write('\n')

    def _format_resource(self, resource, width= -1, show_empty=False, resource_context=None, mode=FormatMode.FULL, sort_strategy=['id', 'name', 'description'], parent=None):
        context = self.context
        settings = context.settings
        stdout = context.terminal.stdout
        fields_exceptions = ['link', 'href', 'parentclass', '_Base__context']
        reduced_mode_fields = ['id', 'name', 'description']

        fields = self.sort_fields(self._get_fields(resource), sort_strategy)
        width0 = width

        if width0 == -1:
            if mode == FormatMode.FULL:
                width0 = self.__get_max_field_width(resource,
                                                    fields_exceptions,
                                                    width, show_empty,
                                                    resource_context,
                                                    mode=mode)
            else:
                for item in reduced_mode_fields:
                    width0 = max(width0, len(item))

        format0 = '%%-%ds' % width0

        for field in fields:
            if field not in fields_exceptions:

                value = getattr(resource, field)
                if type(value) in TextFormatter.complex_type_exceptions:
                    value = str(value)

                if isinstance(value, list):
                    value_list = value
                    for item in value_list:
                        value = item
                        if hasattr(params, type(value).__name__) or hasattr(brokers, type(value).__name__):
                            if resource_context is not None:
                                self._format_resource(resource=value,
                                                      width=width0,
                                                      show_empty=show_empty,
                                                      resource_context=(resource_context + '.' + field),
                                                      mode=mode,
                                                      parent=resource)
                            else:
                                self._format_resource(resource=value,
                                                      width=width0,
                                                      show_empty=show_empty,
                                                      resource_context=field,
                                                      mode=mode,
                                                      parent=resource)
                            continue
                        if value == None and show_empty == True:
                            value = ''
                        elif value == None: continue
                        self.__write_context(format0, field, value, resource_context, reduced_mode_fields=reduced_mode_fields, mode=mode, parent=parent, resource=resource)
                else:
                    if hasattr(params, type(value).__name__) or hasattr(brokers, type(value).__name__):
                        if resource_context is not None:
                            self._format_resource(resource=value,
                                                  width=width0,
                                                  show_empty=show_empty,
                                                  resource_context=(resource_context + '.' + field),
                                                  mode=mode,
                                                  parent=parent)
                        else:
                            self._format_resource(resource=value,
                                                  width=width0,
                                                  show_empty=show_empty,
                                                  resource_context=field,
                                                  mode=mode,
                                                  parent=parent)
                        continue
                    if value == None and show_empty == True:
                        value = ''
                    elif value == None: continue
                    self.__write_context(format0, field, value, resource_context, reduced_mode_fields=reduced_mode_fields, mode=mode, parent=parent, resource=resource)

    def _get_max_field_width_in_collection(self, collection, mode):
        fields_exceptions = ['link', 'href', 'parentclass', '_Base__context']
        reduced_mode_fields = ['id', 'name', 'description']
        width = -1
        width_static = -1

        if mode != FormatMode.FULL:
            for item in reduced_mode_fields:
                width_static = max(width_static, len(item))
            return width_static

        for resource in collection:
            width_dynamic = -1
            width_dynamic = self.__get_max_field_width(
                            resource,
                            fields_exceptions,
                            mode=mode
            )
            width = max(width, width_dynamic)
        return width

    def _format_collection(self, collection, show_empty=False):
        context = self.context
        stdout = context.terminal.stdout
        mode = FormatMode.REDUCED if not show_empty else FormatMode.FULL
        width = self._get_max_field_width_in_collection(collection, mode)
        for resource in collection:
            if isinstance(resource, Base):
                self._format_resource(resource=resource.superclass,
                                      width=width,
                                      mode=mode)
            else:
                self._format_resource(resource=resource,
                                      width=width,
                                      mode=mode)
            stdout.write('\n')

    def format(self, context, result, show_all=False):
        RESOURCE_EXCEPTIONS = [ApiSummary]
        COLLECTION_EXCEPTIONS = []
        self.context = context

        if isinstance(result, params.BaseResource) or type(result) in RESOURCE_EXCEPTIONS:
            if isinstance(result, Base):
                context.terminal.stdout.write('\n')
                self._format_resource(result.superclass, show_empty=show_all)
                context.terminal.stdout.write('\n')
            else:
                context.terminal.stdout.write('\n')
                self._format_resource(resource=result, show_empty=show_all)
                context.terminal.stdout.write('\n')
        elif isinstance(result, list) or type(result) in COLLECTION_EXCEPTIONS:
            context.terminal.stdout.write('\n')
            self._format_collection(collection=result, show_empty=show_all)
#            context.terminal.stdout.write('\n')



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


_super = super

def super(type, self):
    if issubclass(type, object):
        return _super(type, self)
    class proxy(object):
        def __init__(self, type, obj):
            object.__setattr__(self, '__type__', type)
            object.__setattr__(self, '__obj__', obj)
        def __getattribute__(self, name):
            def bind(func, self):
                def _f(*args):
                    func(self, *args)
                return _f
            type = object.__getattribute__(self, '__type__')
            obj = object.__getattribute__(self, '__obj__')
            return bind(getattr(type, name), obj)
    base = type.__bases__[0]
    return proxy(base, self)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from cli.messages import Messages

import sys
from ovirtcli.utils.versionhelper import VersionHelper


class InfoCommand(OvirtCommand):

    name = 'info'
    description = 'shows environment and application components info'
    helptext = """\
        == Usage ==

        info

        == Description ==

        Shows environment and application components info.
        """

    def execute(self):
        context = self.context

        self.write(
               '\n'
               +
               Messages.Info.BACKEND_VERSION
               %
               VersionHelper.to_string(context.backend_version)
        )
        self.write(
               Messages.Info.SDK_VERSION
               %
               VersionHelper.to_string(context.sdk_version)
        )
        self.write(
               Messages.Info.CLI_VERSION
               %
               VersionHelper.to_string(context.cli_version)
        )
        self.write(
               Messages.Info.PYTHON_VERSION
               %
               VersionHelper.to_string((sys.version_info))
               +
               '\n'
        )
        self.write(
               Messages.Info.BACKEND_ENTRY_POINT
               %
               context.url
        )
        self.write('')



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand


class SummaryCommand(OvirtCommand):

    name = 'summary'
    description = 'displaying the system status'
    helptext = """\
        == Usage ==

        summary

        == Description ==

        Displaying the system status.
        """

    def execute(self):
        connection = self.check_connection()
        self.context.formatter.format(
                  self.context,
                  connection.get_summary()
        )



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper
from cli.messages import Messages


class RemoveCommand(OvirtCommand):

    name = 'remove'
    aliases = ('delete',)
    description = 'removes an object'
    args_check = 2
    valid_options = [ ('*', str) ]

    helptext = """\
        == Usage ==
    
        remove <type> <id> [parent identifiers] [command options]

        == Description ==

        Removes an object. The following arguments are required:

          * type        The type of object to remove
          * id          The object identifier

        Objects can be identified by their name and by their unique id.

        == Supported Help formats ==

        - This help will list all available attribute options for given resource removal
          
          * format      - help remove type
          * example     - help remove storagedomain mydomain

        - This help will list all available attribute options for given subresource removal
          
          * format      - help remove subtype --resource-identifier
          * example     - help remove disk --vm-identifier iscsi_desktop

        == Available Types ==

        The <type> parameter must be one of the following.

          $types

        == Object Identifiers ==

        Some objects can only exist inside other objects. For example, a disk
        can only exist in the content of a virtual machine. In this case, one
        or more object identifier opties needs to be provided to identify the
        containing object.

        An object identifier is an option of the form '--<type>id <id>'. This
        would identify an object with type <type> and id <id>. See the
        examples section below for a few examples.

        == Examples ==

        - This example removes a virtual machine named "myvm"

          $ remove vm myvm

        - This example removes the disk "disk0" from the virtual machine named "myvm"

          $ remove disk disk0 --vm-identifier myvm

        - This example removes the storagedomain "mydomain" using host named "myhost"

          $ remove storagedomain mydomain --host-id myhost
          
        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    helptext1 = """\
        == Usage ==

        remove <type> <id> [parent identifiers] [command options]

        == Description ==

        Removes an object with type '$type'. See 'help remove' for generic
        help on deleting objects.

        == Attribute Options ==

        The following options are available for objects with type '$type':

          $options

        == Collection based option format ==

          * [--x-y: collection]
            {
              [y.a: string]
              [y.b: string]
              [y.c: string]
            }

          e.g:

          --x-y "y.a=a1,y.b=b1,y.c=c1"
          --x-y "y.a=a2,y.b=b2,y.c=c2"
          ...

          where a, b, c are option properties and aN, bN, cN is actual user's data

        == Return Values ==

          $statuses
        """

    def execute(self):
        """Execute "remove"."""
        args = self.arguments
        opts = self.options

        typs = TypeHelper.get_types_containing_method(
            RemoveCommand.aliases[0],
            expendNestedTypes=True,
            groupOptions=True
        )

        resource = self.get_object(
            args[0],
            args[1],
            self.resolve_base(opts),
            context_variants=typs[args[0]]
        )

        if resource is None:
            self.error(
               Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
            )
        elif hasattr(resource, RemoveCommand.aliases[0]):
            result = self.execute_method(
               resource, RemoveCommand.aliases[0], opts
            )
        else:
            self.error(
               Messages.Error.OBJECT_IS_IMMUTABLE % (args[0], args[1])
            )

        if not result:
            self.write(Messages.Info.ACCEPTED)
        else:
            self.context.formatter.format(self.context, result)

    def show_help(self):
        """Show help for "remove"."""
        self.check_connection()
        args = self.arguments
        opts = self.options

        subst = {}
        types = TypeHelper.get_types_containing_method(
               RemoveCommand.aliases[0],
               expendNestedTypes=True,
               groupOptions=True
        )

        if not args or self.is_supported_type(types.keys(), args[0]):
            subst['types'] = self.format_map(types)
            statuses = self.get_statuses()
            subst['statuses'] = self.format_list(statuses)

            if len(args) == 2:
                base = self.resolve_base(self.options)
                obj = self.get_object(
                          args[0],
                          args[1],
                          base,
                          context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                          Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )

                helptext = self.helptext1
                params_list = self.get_options(
                           method=RemoveCommand.aliases[0],
                           resource=obj,
                           sub_resource=base,
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)
                subst['type'] = args[0]

            elif len(args) == 1 and len(opts) == 2:
                helptext = self.helptext1
                subst['type'] = args[0]

                options = self.get_options(
                           method=RemoveCommand.aliases[0],
                           resource=args[0],
                           sub_resource=self.resolve_base(self.options),
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(options)
                subst['type'] = args[0]
            elif len(args) == 1:
                helptext = self.helptext
                subst['type'] = args[0]
                subst['types'] = self.format_map({args[0]:types[args[0]]})
            else:
                helptext = self.helptext

            helptext = self.format_help(helptext, subst)
            self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import sys
import textwrap
from optparse import OptionParser

from cli.object import create
from cli.context import ExecutionContext
from cli.command import *


class TestContext(ExecutionContext):

    name = 'cli-test'
    welcome = textwrap.dedent("""\
        Welcome to cli-test. This is a test driver for the python-cli.
        Type 'exit' to exit or 'help' for help.
        """)
    goodbye = 'Goodbye!'

    def setup_commands(self):
        """Add commands."""
        self.add_command(SetCommand)
        self.add_command(SaveCommand)
        self.add_command(HelpCommand)
        self.add_command(StatusCommand)
        self.add_command(CdCommand)
        self.add_command(ClearCommand)
        self.add_command(PwdCommand)
        self.add_command(ExitCommand)


def main():
    """Test driver for python-cli."""
    parser = create(OptionParser)
    parser.add_option('-f', '--filter', metavar='FILE',
                      help='execute commands from FILE')
    parser.add_option('-d', '--debug', action='store_true',
                      default=False, help='enable debugging mode')
    parser.add_option('-v', '--verbose', action='store_const',
                      dest='verbosity', default=0, const=10,
                      help='be verbose',)
    opts, args = parser.parse_args()

    if opts.filter:
        try:
            cmdin = file(opts.filter)
        except IOError, e:
            sys.stderr.write('error: %s\n' % e.strerror)
            sys.exit(1)
    else:
        cmdin = sys.stdin

    context = create(TestContext, cmdin)
    context.settings['cli:debug'] = opts.debug
    context.settings['cli:verbosity'] = opts.verbosity

    if len(args) == 0:
        context.execute_loop()
    else:
        command = ' '.join(args) + '\n'
        context.execute_string(command)

    sys.exit(context.status)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.infrastructure.settings import OvirtCliSettings

class Messages():
    class Error():
        OBJECT_IS_IMMUTABLE = '%s "%s" is immutable.'
        NO_SUCH_OPTION = 'option "%s" is not supported, see help for more details.'
        NO_SUCH_OBJECT = '%s %s does not exist.'
        NO_SUCH_ACTION = 'action "%s" does not exist in this context.'
        NO_SUCH_TYPE = 'type "%s" does not exist.'
        NO_SUCH_CONTEXT = 'cannot find any context for type "%s" using given arguments.'
        NO_SUCH_TYPE_OR_ARS_NOT_VALID = 'no such type "%s" or given arguments not valid.'
        NO_IDENTIFIER = '%s identifier required.'
        NO_QUERY_ARGS = '"--query" argument is not available for this type of listing.'
        NO_KWARGS = '"--kwargs" argument is not available for this type of %s.'
        NO_ID = '"--id" argument is not available for this type of %s.'
        NO_NAME = '"--name" argument is not available for this type of show.'
        NO_CONSOLE_FOUND = '%s viewer was not found, please install %s first.'
        NO_SUCH_COMMAND = '%s command was not found, please install it first.'
        NOT_CONNECTED = OvirtCliSettings.PRODUCT.lower() + '-shell is not connected.'
        NO_SUCH_COLLECTION = 'no such collection "%s" or given arguments not valid.'
        NO_CERTIFICATES = 'server CA certificate file must be specified for SSL secured connection.'
        CANNOT_CREATE = 'cannot create "%s" because %s collection is not available or given arguments not valid.'
        CANNOT_CONNECT_TO_VM_DUE_TO_INVALID_STATE = 'cannot connect to vm due to invalid state.'
        CANNOT_START_CONSOLE_CLIENT = '$DISPLAY not set, cannot start a %s client.'
        CANNOT_SET_VM_TICKET = 'could not set a ticket for the vm.'
        CANNOT_CONNECT_TO_BACKEND = 'could NOT reach %s manager\n.'
        CANNOT_CONSTRUCT_COLLECTION_MEMBER_VIEW = 'cannot construct collection/member view, checked variants are:\n%s.\n'
        INVALID_DISPLAY_PROTOCOL = 'display protocol "%s" is not supported.'
        INVALID_COMMAND = 'command "%s" is not valid or not available while not connected.'
        INVALID_ENV_MODE_FOR_CONSOLE = 'not running in a GUI, cannot start a %s viewer.'
        INVALID_OPTION = 'option %s cannot be empty.'
        INVALID_COLLECTION_BASED_OPTION_SYNTAX = 'invalid syntax at "--%s", see help on collection based arguments for more details.'
        INVALID_ARGUMENT_SEGMENT = '"%s" is invalid argument segment.'
        INVALID_OPTION_SEGMENT = '"%s" is invalid segment at option "--%s".'
        INVALID_KWARGS_FORMAT = '"%s" is invalid --kwargs argument, valid format is "x=y;z=q;...".'
        INVALID_KWARGS_CONTENT = '--kwargs constraint cannot be empty.'
        INVALID_ARGUMENT_TYPE = 'value used for the \"%s\" is \"%s\", while %s'
        INVALID_URL_SEGMENT = '"%s" is invalid url format, valid format is http[s]://server:port/path, where path is usually "ovirt-engine/api" or, for older versions of the engine, just "api".'
        MISSING_CONFIGURATION_VARIABLE = 'missing configuration variable: %s.'
        UNSUPPORTED_ATTRIBUTE = 'object construction failure, this could happen if you using unsupported option, please see help for the given command for more details.'
    class Warning():
        CANNOT_FETCH_HOST_CERT_SUBJECT = 'could not fetch host certificate info.'
        CANNOT_FETCH_HOST_CERT_SUBJECT_LEGACY_SDK = 'could not fetch host certificate info cause used backend/sdk does not support it.'
        HOST_IDENTITY_WILL_NOT_BE_VALIDATED = 'host identity will not be validated.'
        ALREADY_CONNECTED = '\nalready connected\n\n'
    class Info():
        POSSIBALE_ARGUMENTS_COMBINATIONS = ',\npossible arguments combinations are %s.'
        POSSIBLE_VM_STATES_FOR_CONSOLE = ',\npossible vm states are %s.'
        ACTION_STATUS = 'action status "%s".'
        STATUS = '\nlast command status: %s'
        PRODUCT_VERSION = "\n%s version: %s.\n"
        NOT_CONNECTED = 'not connected.'
        SUCESS_CONNECT_TO_BACKEND = '\nsuccess: ' + OvirtCliSettings.PRODUCT + ' manager could be reached OK.\n\n'
        SDK_VERSION = 'sdk version    : %s'
        CLI_VERSION = 'cli version    : %s'
        BACKEND_VERSION = 'backend version: %s'
        BACKEND_ENTRY_POINT = 'entry point    : %s'
        PYTHON_VERSION = 'python version : %s'
        PLATFORM = 'platform       : %s'
        ACCEPTED = '\naccepted.\n'



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import re
from xml.dom import minidom

from ovirtcli.format.format import Formatter

# This module is called xml_ to prevent a naming conflict with the standard
# libary.


class XmlFormatter(Formatter):
    """XML formatter."""

    name = 'xml'

    _re_spacing1 = re.compile('(>)\n\s+([^<\s])')
    _re_spacing2 = re.compile('([^>])\n\s+(<)')

    def format(self, context, result, scope=None):
        if not hasattr(result, 'toxml'):
            raise TypeError, 'Expecting a binding instance.'
        self.context = context
        stdout = context.terminal.stdout
        buf = result.toxml()
        xml = minidom.parseString(buf)
        buf = xml.documentElement.toprettyxml(indent='  ')
        # XXX: These strings substituations are not context-sensitive
        # and may introduce issues. It removes illegal spacing that
        # .toprettyxml() is adding.
        buf = self._re_spacing1.sub(r'\1\2', buf)
        buf = self._re_spacing2.sub(r'\1\2', buf)
        stdout.write(buf)




from ovirtcli.command.action import ActionCommand
from ovirtcli.command.add import AddCommand
from ovirtcli.command.connect import ConnectCommand
from ovirtcli.command.console import ConsoleCommand
from ovirtcli.command.remove import RemoveCommand
from ovirtcli.command.disconnect import DisconnectCommand
from ovirtcli.command.help import HelpCommand
from ovirtcli.command.list import ListCommand
from ovirtcli.command.ping import PingCommand
from ovirtcli.command.show import ShowCommand
from ovirtcli.command.status import StatusCommand
from ovirtcli.command.update import UpdateCommand
from ovirtcli.command.history import HistoryCommand



#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.annotations.requires import Requires
from ovirtcli.listeners.ilistener import IListener
from ovirtcli.events.abstractevent import AbstractEvent

class Event(AbstractEvent):
    '''
    The event class implementation
    '''

    def __init__(self):
        self.__listeners = []
        self.__id = id(self)

    @Requires(IListener)
    def __iadd__(self, listener):
        """
        adds event IListener

        @param listener: listener to register
        """
        assert listener != None
        self.__listeners.append(listener)
        return self

    @Requires(IListener)
    def __isub__(self, listener):
        """
        removes event IListener

        @param listener: listener to unregister
        """
        assert listener != None
        self.__listeners.remove(listener)
        return self

    def fire(self, *args, **keywargs):
        '''
        event listeners notification

        @param args: a list o args
        @param kwargs: a list o kwargs
        '''
        for listener in self.__listeners:
            listener.onEvent(*args, **keywargs)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.listeners.abstractlistener import AbstractListener

class ErrorListener(AbstractListener):
    '''
    Listens for the error events
    '''

    def __init__(self, shell):
        """
        @param shell: EngineShell instance
        """
        assert shell != None
        self.__shell = shell

    def onEvent(self, *args, **kwargs):
        '''
        fired when error event is raised

        @param args: a list o args
        @param kwargs: a list o kwargs
        '''
        # TODO: log error




import os
import sys

from distutils.command.build import build
from setuptools import setup, Command


version_info = {
    'name': 'ovirt-shell',
    'version': '3.5.0.4',
    'description': 'A command-line interface to oVirt Virtualization',
    'author': 'Michael Pasternak',
    'author_email': 'mpastern@redhat.com',
    'url': 'http://www.ovirt.org/wiki/CLI',
    'license': 'ASL2',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.4' ],
}


setup(
    package_dir={ '': 'src' },
    packages=[ 'ovirtcli', 'ovirtcli.command', 'ovirtcli.format',
                 'ovirtcli.platform', 'ovirtcli.platform.posix',
                 'ovirtcli.platform.windows', 'ovirtcli.shell', 'ovirtcli.utils', 'cli',
                 'cli.command', 'cli.platform', 'cli.platform.posix', 'ovirtcli.infrastructure',
                 'ovirtcli.annotations', 'ovirtcli.events', 'ovirtcli.listeners', 'ovirtcli.meta',
                 'ovirtcli.state'],
    install_requires=[ 'ovirt-engine-sdk-python >= 3.5.0.1', 'ply >= 3.3', 'kitchen >= 1' ],
    entry_points={ 'console_scripts': [ 'ovirt-shell = ovirtcli.main:main' ] },
    **version_info
)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.shell.cmdshell import CmdShell
from ovirtcli.utils.typehelper import TypeHelper
from ovirtcli.utils.autocompletionhelper import AutoCompletionHelper
from ovirtsdk.infrastructure import brokers
from ovirtcli.utils.methodhelper import MethodHelper


class UpdateCmdShell(CmdShell):
    NAME = 'update'

    def __init__(self, context, parser):
        CmdShell.__init__(self, context, parser)

    def do_update(self, args):
        return self.context.execute_string(UpdateCmdShell.NAME + ' ' + args + '\n')

    def __add_resource_specific_options(self, obj, specific_options, line, key=None):
        obj_type = TypeHelper.getDecoratorType(TypeHelper.to_singular(obj))
        if obj_type and hasattr(brokers, obj_type):
            obj_typ_ref = getattr(brokers, obj_type)
            if obj_typ_ref and hasattr(obj_typ_ref, UpdateCmdShell.NAME):
                method_args = MethodHelper.get_documented_arguments(method_ref=getattr(obj_typ_ref, UpdateCmdShell.NAME),
                                                                    as_params_collection=True,
                                                                    spilt_or=True)

                if method_args:
                    specific_options[key if key is not None else obj] = method_args

    def complete_update(self, text, line, begidx, endidx):
        args = TypeHelper.get_types_containing_method(UpdateCmdShell.NAME, expendNestedTypes=True)
        specific_options = self.get_resource_specific_options(args,
                                                              line,
                                                              callback=self.__add_resource_specific_options)

        return AutoCompletionHelper.complete(line, text, args, specific_options=specific_options)

    def is_update_argument(self, line, key):
        args = TypeHelper.get_types_containing_method(UpdateCmdShell.NAME, expendNestedTypes=True)
        if key in args:
            return True
        specific_options = self.get_resource_specific_options(args,
                                                              line,
                                                              callback=self.__add_resource_specific_options)
        for arg_key in specific_options.keys():
             if key in specific_options[arg_key]:
                 return True
        return False



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.platform import vnc, spice
from ovirtcli.command.command import OvirtCommand
from cli.messages import Messages


class ConsoleCommand(OvirtCommand):

    name = 'console'
    description = 'open a console to a VM'
    args_check = 1

    helptext = """\
        == Usage ==

        console <vm>

        == Description ==

        This opens up a graphical console to a virtual machine. Depending on
        the virtual machine display, this will fire up an external VNC or
        SPICE client.
        """

    def execute(self):
        self.check_connection()
        args = self.arguments
        CONSOLE_STATES = ['powering_up', 'up', 'reboot_in_progress']
        host_subject = ''

        vm = self.get_object('vm', args[0])
        if vm is None:
            self.error(
               Messages.Error.NO_SUCH_OBJECT % ('vm', args[0])
            )
        if vm.status.state not in CONSOLE_STATES:
            self.error(
               Messages.Error.CANNOT_CONNECT_TO_VM_DUE_TO_INVALID_STATE +
               Messages.Info.POSSIBLE_VM_STATES_FOR_CONSOLE % str(CONSOLE_STATES)
           )

        host_addr = vm.display.address
        proto = vm.display.type_
        port = vm.display.port
        secport = vm.display.secure_port
        action = vm.ticket()

        if action.status.state != 'complete':
            self.error(
               Messages.Error.CANNOT_SET_VM_TICKET
            )
        ticket = action.ticket.value
        debug = self.context.settings['cli:debug']

        if proto == 'vnc':
            vnc.launch_vnc_viewer(host_addr, port, ticket, debug)
        elif proto == 'spice':
            certurl = '%s/ca.crt' % (self.context.url.replace('/api', ''))

            if vm.host and vm.host.id:
                host = self.get_object('host', vm.host.id)
                if host:
                    if hasattr(host, 'certificate') and host.certificate:
                        if host.certificate.subject:
                            host_subject = host.certificate.subject
                        else:
                            self.warning(
                                 Messages.Warning.CANNOT_FETCH_HOST_CERT_SUBJECT
                            )
                    else:
                        self.warning(
                             Messages.Warning.CANNOT_FETCH_HOST_CERT_SUBJECT_LEGACY_SDK
                        )
            if host_subject == '':
                    self.warning(
                         Messages.Warning.HOST_IDENTITY_WILL_NOT_BE_VALIDATED
                    )
            spice.launch_spice_client(
                          host_addr,
                          host_subject,
                          port, secport,
                          ticket,
                          certurl,
                          vm.name,
                          debug
            )
        else:
            self.error(
                   Messages.Error.INVALID_DISPLAY_PROTOCOL % proto
            )



#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.state.dfsastate import DFSAState
from ovirtcli.annotations.requires import Requires
import types

class DFSAEvent(object):
    '''
    Finite-State Automata event
    '''

    def __init__(self, name, sources, destination, description=None, callbacks=[]):
        '''
        @param name: the state event name
        @param sources: the source states from which destination state is eligible
        @param destination: the destination state
        @param callbacks: collection of callbacks to invoke on this event (TODO)
        '''
        self.__id = id(self)
        self.__name = name
        self.__sources = sources
        self.__destination = destination
        self.__description = description
        self.__callbacks = callbacks

    def get_name(self):
        """
        @return: the name of DFSAEvent
        """
        return self.__name

    def get_sources(self):
        """
        @return: the sources of DFSAEvent
        """
        return self.__sources

    def get_destination(self):
        """
        @return: the destination of DFSAEvent
        """
        return self.__destination

    def get_description(self):
        """
        @return: the description of DFSAEvent
        """
        return self.__description

    def get_callbacks(self):
        """
        @return: the destination of DFSAEvent
        """
        return self.__callbacks

    @Requires(types.MethodType)
    def add_callback(self, callback):
        """
        adds new callback to event

        @param callback: the method to register
        """
        if self.__callbacks == None:
            self.__callbacks = []
        self.__callbacks.append(callback)

    def __str__(self):
        return (
            "DFSAEvent: %s\n" + \
            "name: %s\n" + \
            "sources: %s\n" + \
            "destination: %s\n" + \
            "callbacks: %s") % (
                 str(self.__id),
                 self.get_name(),
                 str([
                      str(DFSAState(src))
                      for src in self.get_sources()
                    ]
                 ),
                 str(DFSAState(self.get_destination())),
                 str([str(callback) for callback in self.get_callbacks()])
             )



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os

import termios
import struct
import curses

from cli.terminal import Terminal

#TODO: check it the same import
#import fcntl
from mailbox import fcntl


class PosixTerminal(Terminal):
    """A terminal for Posix style systems."""

    def __init__(self, *args):
        super(PosixTerminal, self).__init__(*args)
        try:
            self._tty = os.open('/dev/tty', os.O_RDWR)
            curses.setupterm()
        except OSError:
            self._tty = None

    def _get_width(self):
        if self._tty:
            packed = fcntl.ioctl(self._tty, termios.TIOCGWINSZ, 'xxxx')
            width = struct.unpack('@HH', packed)[1]
        else:
            width = 80
        return width

    width = property(_get_width)

    def _get_height(self):
        if self._tty:
            packed = fcntl.ioctl(self._tty, termios.TIOCGWINSZ, 'xxxx')
            height = struct.unpack('@HH', packed)[0]
        else:
            height = 25
        return height

    def clear(self):
        if self._tty:
            os.system("clear")

    def set_echo(self, echo):
        if self._tty:
            attrs = termios.tcgetattr(self._tty)
            if echo:
                attrs[3] |= termios.ECHO
            else:
                attrs[3] &= ~termios.ECHO
            termios.tcsetattr(self._tty, termios.TCSANOW, attrs)

    def close(self):
        if self._tty:
            os.close(self._tty)

    def readline(self, prompt):
        line = raw_input(prompt)
        line += '\n'
        return line
